<?php
/**
 * Copyright © 2017 Portonics. All rights reserved.

 */
namespace PortWallet\PortWallet\Model;

use Magento\Sales\Model\Order\Payment;
use Magento\Sales\Model\Order\Payment\Transaction;

class PortPay extends \Magento\Payment\Model\Method\AbstractMethod
{
    
    protected $_code = 'portpay';
    protected $_canUseForMultishipping  = false;
    protected $_isGateway = true;
    protected $_canAuthorize = true;
    protected $_canCapture = true;
    protected $_canCapturePartial = false;
    protected $_canCaptureOnce = true;
    protected $_canRefund = false;
    protected $_canVoid = false;
    protected $_canUseInternal = true;
    protected $_canUseCheckout = true;
    protected $_isInitializeNeeded = true;
    protected $_params;
    

    public function __construct(
    \Magento\Framework\UrlInterface $urlBuilder,
    \Magento\Framework\Exception\LocalizedExceptionFactory $exception,
    \Magento\Sales\Api\TransactionRepositoryInterface $transactionRepository,
    \Magento\Sales\Model\Order\Payment\Transaction\BuilderInterface $transactionBuilder,
    \Magento\Sales\Model\OrderFactory $orderFactory,
    \Magento\Store\Model\StoreManagerInterface $storeManager,
    \Magento\Sales\Model\Order\Email\Sender\OrderSender $orderSender,
    
   
    \Magento\Framework\Model\Context $context,
    \Magento\Framework\Registry $registry,
    \Magento\Framework\Api\ExtensionAttributesFactory $extensionFactory,
    \Magento\Framework\Api\AttributeValueFactory $customAttributeFactory,
    \Magento\Payment\Helper\Data $paymentData,
    \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig,
    \Magento\Payment\Model\Method\Logger $logger,
    \Magento\Checkout\Model\Session $checkoutSession,
    \Magento\Framework\Model\ResourceModel\AbstractResource $resource = null,
    \Magento\Framework\Data\Collection\AbstractDb $resourceCollection = null,
    array $data = []

    ) 
    {
        $this->_urlBuilder = $urlBuilder;
        $this->_exception = $exception;
        $this->_transactionRepository = $transactionRepository;
        $this->_transactionBuilder = $transactionBuilder;
        $this->_orderSender=$orderSender;
        
        $this->_orderFactory = $orderFactory;
        $this->_storeManager = $storeManager;
        $this->_checkoutSession = $checkoutSession;
      


        parent::__construct(
            $context,
            $registry,
            $extensionFactory,
            $customAttributeFactory,
            $paymentData,
            $scopeConfig,
            $logger,
            $resource,
            $resourceCollection,
            $data
        );
        
    }

    public function initialize($paymentAction, $stateObject)
    {
        $payment = $this->getInfoInstance();

        $order = $payment->getOrder();
        $order->setCanSendNewEmailFlag(false);
        $order->setCustomerNoteNotify(false);

        $stateObject->setState(\Magento\Sales\Model\Order::STATE_NEW);
        $stateObject->setStatus('new');
        $stateObject->setIsNotified(false);
    }

    private function createTransaction($order = null, $paymentData = array())
    {
        try {
            //get payment object from order object
            $payment = $order->getPayment();
            $payment->setLastTransId($paymentData['id']);
            $payment->setTransactionId($paymentData['id']);
            $payment->setAdditionalInformation(
                [\Magento\Sales\Model\Order\Payment\Transaction::RAW_DETAILS => (array) $paymentData]
            );
            $formatedPrice = $order->getBaseCurrency()->formatTxt(
                $order->getGrandTotal()
            );
 
            $message = __('The authorized amount is %1.', $formatedPrice);
            //get the object of builder class
            $trans = $this->_transactionBuilder;
            $transaction = $trans->setPayment($payment)
            ->setOrder($order)
            ->setTransactionId($paymentData['id'])
            ->setAdditionalInformation(
                [\Magento\Sales\Model\Order\Payment\Transaction::RAW_DETAILS => (array) $paymentData]
            )
            ->setFailSafe(true)
            //build method creates the transaction and returns the object
            ->build(\Magento\Sales\Model\Order\Payment\Transaction::TYPE_CAPTURE);
 
            $payment->addTransactionCommentsToOrder(
                $transaction,
                $message
            );
            $payment->setParentTransactionId(null);
            $payment->save();
            $order->setTotalPaid($order->getGrandTotal());
            $order->save();
             
            return  $transaction->save()->getTransactionId();
        } catch (Exception $e) {
            //log errors here
        }
    }

    public function getCheckoutUrl($order, $storeId = null)
    {
       
        $customerData = $order->getBillingAddress();
        $orderId;
        $productdescription = "";
        $total=$order->getBaseGrandTotal();
        
        $items = $order->getAllItems();
        
        foreach ($items as $item) {
            
            $productdescription .= (int)$item->getQtyOrdered() . " X " . $item->getName() . "<br/>";
            $orderId=$item->getOrderId();
        }

        
        $sandbox = $this->_scopeConfig->getValue('payment/portpay/sandbox', \Magento\Store\Model\ScopeInterface::SCOPE_STORE);
        

        if ($sandbox) {
            $payment_url = "https://payment-sandbox.portwallet.com/payment/";
            $app_key = $this->_scopeConfig->getValue('payment/portpay/sandbox_api_key', \Magento\Store\Model\ScopeInterface::SCOPE_STORE);
            $secret_key =$this->_scopeConfig->getValue('payment/portpay/sandbox_api_secret', \Magento\Store\Model\ScopeInterface::SCOPE_STORE);
        } else {
            $payment_url = "https://payment.portwallet.com/payment/";
            $app_key = $this->_scopeConfig->getValue('payment/portpay/api_key', \Magento\Store\Model\ScopeInterface::SCOPE_STORE);
            $secret_key = $this->_scopeConfig->getValue('payment/portpay/api_secret', \Magento\Store\Model\ScopeInterface::SCOPE_STORE);
        }
        $data = array();
        $data['app_key'] = $app_key;
        $data['timestamp'] = time();
        $data['token'] =  md5($secret_key . $data['timestamp']);
        $data['amount'] = $total;
        $data['currency'] = "BDT";
        $data['product_name'] = "Magento";
        $data['product_description'] = $productdescription;

        $data['name'] =$customerData['firstname'] . ' ' . $customerData['lastname'];;
        $data['email'] = $customerData['email'];
        $data['phone'] = $customerData['telephone'];
        $data['address'] = $customerData['street'];
        $data['city'] = $customerData['city'];
        $data['state'] = "NA";
        $data['zipcode'] = $customerData['postcode'];
        $data['country'] = "BD";
        $data['orderId'] = $orderId;
        $data['redirect_url'] = $this->_urlBuilder->getUrl("pay/index/ipn/", ['_secure' => true]);
        $data['ipn_url'] = $this->_urlBuilder->getUrl("pay/index/ipn/", ['_secure' => true]);
        $response = $this->generateInvoice($data);
        
        if($response->status == 200)
        {
            $redirect =  $payment_url."?invoice=".$response->data->invoice_id;
            return array("url" => $redirect);
           
        }die();
    }
    public function createTransactionIpn($data,$order){
             $order = $order->load($data->orderId);
             $paymentData=[
                    'id' => $data->invoice_id
             ];
          
            if($order->canInvoice()) {
                     $order->setItems($order->getAllItems());
                     $invoice = $order->prepareInvoice();
                     $invoice->setShippingAmount($order->getShippingAmount());
                     $invoice->setSubtotal($order->getSubtotal());
                     $invoice->setBaseSubtotal($order->getBaseSubtotal());
                     $invoice->setGrandTotal($order->getGrandTotal());
                     $invoice->setBaseGrandTotal($order->getBaseGrandTotal());
                     $invoice->register();
                     $invoice->setState(2);
                     $invoice->save();
                     $this->_orderSender->send($order);
                     $this->createTransaction($order,$paymentData);
                     
              }
    }
    private function generateInvoice($data)
    {

        $data['call'] = "gen_invoice";
        $this->_params = $data;
        
        return $this->_process("post");
    }

    public function ipnValidate($data) {
        $data['call'] = "ipn_validate";
        $this->_params = $data;

        return $this->_processIpn("post");
    }

    private function _process($method = "get") {
        return $this->_request($method);
    }

   private function _processIpn($method = "get") {

        $sandbox = $this->_scopeConfig->getValue('payment/portpay/sandbox', \Magento\Store\Model\ScopeInterface::SCOPE_STORE);
        if ($sandbox) {
            $app_key = $this->_scopeConfig->getValue('payment/portpay/sandbox_api_key', \Magento\Store\Model\ScopeInterface::SCOPE_STORE);
            $secret_key =$this->_scopeConfig->getValue('payment/portpay/sandbox_api_secret', \Magento\Store\Model\ScopeInterface::SCOPE_STORE);
        } else {
            $app_key = $this->_scopeConfig->getValue('payment/portpay/api_key', \Magento\Store\Model\ScopeInterface::SCOPE_STORE);
            $secret_key = $this->_scopeConfig->getValue('payment/portpay/api_secret', \Magento\Store\Model\ScopeInterface::SCOPE_STORE);
        }

        $this->_params['timestamp'] = time();
        $this->_params['app_key'] = $app_key;
        $this->_params['token'] = md5($secret_key . $this->_params['timestamp']);
        return $this->_request($method);
    }
    
    private function _request($method) {
    
        $ch = curl_init();
    
        $sandbox = $this->_scopeConfig->getValue('payment/portpay/sandbox', \Magento\Store\Model\ScopeInterface::SCOPE_STORE);

        if ($sandbox) {
            $api_url = "http://api-sandbox.portwallet.com/api/v1/";

        } else {
            $api_url = "http://api.portwallet.com/api/v1/";
            
        }
      
       curl_setopt($ch, CURLOPT_URL, $api_url); 
        
        if ($method == "post") {
            curl_setopt($ch, CURLOPT_POST, true);
            curl_setopt($ch, CURLOPT_POSTFIELDS, $this->_params);
        }

        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        $response = curl_exec($ch);
        $info = curl_getinfo($ch);
        curl_close($ch);
        if ($info['http_code'] == 200 && !empty($response)) {
            return json_decode($response);
        }
        else
            return false;
    }

}
